//
//  tank
//
//  Import and Playlist Daemon for autoradio project
//
//
//  Copyright (C) 2017-2019 Christian Pointner <equinox@helsinki.at>
//
//  This file is part of tank.
//
//  tank is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  any later version.
//
//  tank is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with tank. If not, see <http://www.gnu.org/licenses/>.
//

package auth

import (
	"net/http"
	"strings"

	"github.com/gorilla/mux"
)

var auth = &Auth{}

type Auth struct {
	sessions *SessionManager
	oidc     *OIDCBackend
}

func Init(c *Config) (err error) {
	if c == nil {
		// authentication is disabled
		return
	}

	if auth.sessions, err = NewSessionManager(c.Sessions); err != nil {
		return
	}

	if c.OIDC != nil {
		if auth.oidc, err = NewOIDCBackend(c.OIDC); err != nil {
			return
		}
	}

	return
}

func login(w http.ResponseWriter, r *http.Request) {
	method := r.URL.Query().Get("method")
	switch strings.ToLower(method) {
	case "oidc":
		if auth.oidc == nil {
			http.Error(w, "OIDC authentication is not configured", http.StatusBadRequest)
			return
		}
		auth.oidc.HandleLogin(w, r)
	case "":
		http.Error(w, "default/fallback authentication method has not been implemented yet!", http.StatusNotImplemented)
	default:
		http.Error(w, "invalid authentication method: "+method, http.StatusBadRequest)
	}
}

func logout(w http.ResponseWriter, r *http.Request) {
	_, s := getSessionFromCookie(r)
	invalidateSessionCookie(w)
	if s == nil {
		http.Error(w, "Request does not contain a valid session cookie or session is already expired.", http.StatusUnauthorized)
		return
	}
	http.Error(w, "You are now logged out.", http.StatusOK)
}

func disabled(w http.ResponseWriter, r *http.Request) {
	http.Error(w, "authentication is disabled", http.StatusNotAcceptable)
}

func InstallHTTPHandler(r *mux.Router) {
	if auth.sessions == nil {
		r.PathPrefix("/").HandlerFunc(disabled)
		return
	}

	r.HandleFunc("/login", login)
	r.HandleFunc("/logout", logout)
	if auth.oidc != nil {
		r.Handle("/oidc/callback", auth.oidc.CallbackHandler())
	}
}
